<?php
/*
Plugin Name: WP-Sentinel
Plugin URI: http://www.evilsocket.net/
Version: 1.0.3
Author: Simone Margaritelli aka evilsocket
Description: WordPress security system.
*/
/***************************************************************************
 *   @brief WP-Sentinel - Wordpress Security System .                      *
 *   @author Simone Margaritelli (aka evilsocket) <evilsocket@gmail.com>   *
 *                       		                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

add_action( 'init', 	  'wp_sentinel_run', 0 );
add_action( 'admin_menu', 'wp_sentinel_menu' );

include_once( "php/rules.php" );
include_once( "php/config.php" );
include_once( "php/report.php" );
include_once( "php/log.php" );

/*
 * Create and preprocess requests environment for further elaborations.
 */
function wp_sentinel_env_preprocessor(){
	$env = array( "GET"    => $_GET,
	              "POST"   => $_POST,
				  "COOKIE" => $_COOKIE,
				  "USER"   => wp_get_current_user() );
				  
	foreach( array( "GET", "POST", "COOKIE" ) as $scope ){
		$data = $env[$scope];
		foreach( $data as $name => $value ){
			/* if magic quotes are on, remove slashes from value */
			if( function_exists('get_magic_quotes_gpc') && get_magic_quotes_gpc() ){
				$value = stripslashes($value);
			}
			/* urldecode the value */
			$value = @urldecode($value);
			/* trim it */
			$value = trim($value);
			/* update the environment */
			$env[$scope][$name] = $value;			
		}
	}
	
	return $env;
}

/*
 * Perform cross site request forgery check upon admin panel.
 */
function wp_sentinel_csrf_check( $env, $report ){
	/* admin panel ? */
	if( preg_match( "/\/wp\-admin\//i", $_SERVER["REQUEST_URI"] ) ){
		/* check if there's a POST request */
		if( count( array_diff( array_values($env['POST']), array('') ) ) ){
			$valid_referer = get_bloginfo('wpurl');
			$rule          = "^".preg_quote( $valid_referer, '/' );
			/* referer not set or not allowed to POST to the admin panel */
			if( isset($_SERVER['HTTP_REFERER']) == false || preg_match( "/$rule/i", $_SERVER['HTTP_REFERER'] ) == false ){
				$report->addAlarm( "HTTP_REFERER", 
				                   "HTTP_REFERER", 
								   $_SERVER['HTTP_REFERER'], 
								   "invalid or null referer for wordpress administration panel request", 
								   $rule );
			}
		}
	}
}

/*
 * Perform anti-flood checking.
 */
function wp_sentinel_is_flooding(){
	global $wps_config;
	
	$tm_file = ABSPATH."wp-content/plugins/wp-sentinel/log/".$_SERVER['REMOTE_ADDR'].".timestamp";
	
	if( file_exists($tm_file) ){
		$timestamp = (INT)trim( file_get_contents($tm_file) );
		$now	   = time();
		$delay	   = $now - $timestamp;
		/* yep, we are getting flooded! */
		if( $delay < $wps_config['flood_time'] ){
			return true;
		}
	}

	/* create or update timestamp file for anti-flood measure */
	file_put_contents( $tm_file, time() );
	
	return false;
}
 
/*
 * Upon XSS rule, perform tag checking.
 */
function wp_sentinel_allowed_tags( $rule, $value ){
	if( preg_match_all( "/<\s*\/?\s*([^ >]+)[^>]*>/m", $value, $tags ) ){
		$tags		 = $tags[1];
		/* any unallowed tags? */
		if( array_intersect( $tags, $rule['allowed'] ) != $tags ){
			return false;
		}
	}
	return true;
}

function wp_sentinel_allowed_file( $rule, $value ){
	$remotefile = trim($value);
	$hash		= md5($remotefile);
	$cachedfile = ABSPATH."wp-content/plugins/wp-sentinel/vectors/$hash.dat";
	$content	= null;
	
	/* file already cached :) */
	if( file_exists($cachedfile) ){
		$content = @file_get_contents($cachedfile);
	}
	/* fetch the file and add it to the cache */
	else{
		$content = @file_get_contents($remotefile);
		@file_put_contents( $cachedfile, $content );
	}
			
	return !preg_match( $rule['classifier'], $content );
}

/*
 * Main plugin function, ideally run by WP before everything else.
 */
function wp_sentinel_run(){	
	global $wps_config, $wps_rules;
	
	if( $wps_config['enabled'] == 0 ){
		return;
	}
	
	$env 	= wp_sentinel_env_preprocessor();	
	$report = new WPS_Report( $env["USER"] );
					
	/* check for CSRF on admin panel but skip rule checks if user is not a normal user (admin, contributor, editor, ...) */
	if( isset($env["USER"]->user_level) ){
		wp_sentinel_csrf_check( $env, $report );
	}
	else{
		/* basic rules check for normal users (both logged and not logged) */				
		foreach( array( "GET", "POST", "COOKIE" ) as $scope ){
			$data = $env[$scope];
			/* loop each request variable */
			foreach( $data as $name => $value ){
				/* skip whitelisted variables */
				if( in_array( $name, $wps_config['whitelist'] ) ){
					continue;
				}
				/* loop each rule */
				foreach( $wps_rules as $rule ){
					/* do we have a match? */
					if( preg_match( "/{$rule[pattern]}/i", $value ) ){
						/* handle special cases */
						switch( $rule['label'] ){
							/* if it's an XSS rule, extract tags from the request and check if they are allowed */
							case "XSS" : $alarm_state = !wp_sentinel_allowed_tags( $rule, $value ); break;
							/* on RFI, try to classify remote file */
							case "RFI" : $alarm_state = !wp_sentinel_allowed_file( $rule, $value ); break;			
							/* no special cases, just set the alarm state from the main regex */				
							default:
								$alarm_state = true;
						}
						/* finally, add the alarm to the queue if any */
						if( $alarm_state == true ){
							$report->addAlarm( $scope, $name, $value, $rule["type"], $rule["pattern"] );
						}
					}
				}
			}
		}
	}
	
	if( $report->hasAlarms() ){
		/* log and notify only if we are not getting flooded */
		if( wp_sentinel_is_flooding() == false ){	
			/* handle logging */
			if( $wps_config['logging'] == 1 ){
				$log = new WPS_Log();
				foreach( $report->alarms() as $alarm ){		
					$log->log( $env['USER'], $alarm['scope'], $alarm['variable'], $alarm['content'], $alarm['alarm'], $alarm['rule'] );
				}
			}
			/* handle email notification */
			if( $wps_config['notification'] == 1 ){
				$admin   = get_bloginfo("admin_email");
				$message = $report->emailReport();
				$subject = "WP-Sentinel Alarm Report";
				$headers = "From: $admin\r\n" .
				           "X-Mailer: PHP/".phpversion();
	
				@mail( $admin, $subject, $message, $headers );
			}
		}
		
		die( $report->userReport() );
	}
}

/*
 * Little routine to get UTF-8 encoded data from layout files.
 */
function file_get_contents_utf8($filename) { 
     return utf8_decode(file_get_contents($filename));
} 

/*
 * Generate admin panel entry tabs.
 */
function wp_sentinel_generate_tabs(){
	$tabs = array( 'daily'    => 'Daily Logs',
				   'layouts'  => 'Layouts', 
				   'settings' => 'Settings' );
					  
	$active  = isset($_GET['tab']) ? $_GET['tab'] : 'daily';
	$active  = in_array( $active, array_keys($tabs) ) ? $active : 'daily';
	
	echo '<div id="nav">
		  	<h2 class="themes-php" style="border-bottom: 1px solid #CCC; padding: 0px;">';
	foreach( $tabs as $key => $label ){
		echo '<a class="nav-tab '.($active == $key ? 'nav-tab-active' : '').'" href="?page=wp-sentinel&tab='.$key.'">'.$label.'</a>';
	}
	echo '  </h2>
		  </div>';
}

/*
 * Handle the daily report tab. 
 */
function wp_sentinel_handle_daily(){
	/* first of all, do some writability checking */
	$checks = array( 'log', 'vectors', 'php/config.php' );
	foreach( $checks as $check ){
		$item = ABSPATH."wp-content/plugins/wp-sentinel/$check";
		if( !is_writable($item) ){
			echo "<div style='background-color: white; color: red; font-size: 13px; padding: 10px; border: 1px dotted gray;'><b>'$item'</b> is not writable!!!.</div><br>";
		}
	}
	
	$daily_log_path = ABSPATH."wp-content/plugins/wp-sentinel/log/".strftime("log_%y%m%d.xml.php");
		
	if( isset($_GET['wipe']) ){
		if( $_GET['wipe'] == "daily" ){
			unlink($daily_log_path);
		}
		else if( $_GET['wipe'] == "full" ){
			$handle = opendir(ABSPATH."wp-content/plugins/wp-sentinel/log/");
    		while (false !== ($file = readdir($handle))) {
    			if( preg_match( "/log_[\d]+\.xml\.php/", $file ) ){
    				unlink( ABSPATH."wp-content/plugins/wp-sentinel/log/$file" );
    			}
        	}
    		closedir($handle);			
		}
	}
	
?>
<div style="background-color: white; border: 1px dotted gray; padding: 10px;">
	<b>Download</b> : <a href='<?= "/wp-content/plugins/wp-sentinel/logdownload.php?action=daily"; ?>'>daily</a> - <a href='<?= "/wp-content/plugins/wp-sentinel/logdownload.php?action=full"; ?>'>full</a>
	<div style="float:right">
	<b>Wipe</b> : <a href='?page=wp-sentinel&tab=daily&wipe=daily'>daily</a> - <a href='?page=wp-sentinel&tab=daily&wipe=full'>full</a>
	</div>
</div>
<?php	
	if( file_exists($daily_log_path) ){
		$lines = file($daily_log_path);	
?>
<br>
<table width="100%" cellpadding="0" cellspacing="1" style="background-color: white; border: 1px dotted gray; padding: 0;">
	<tr><td>
<b>Logged <?= (count($lines) - 2) / 12 ;?> attacks.</b>
<table width="100%" cellpadding="0" cellspacing="1" style="background-color: white; border: 1px dotted gray; padding: 0;">
<tr>
	<td style="font-weight: bold; background-color: #CCC; padding:3px;" align='center'>Time</th>
	<td style="font-weight: bold; background-color: #CCC; padding:3px;" align='center'>Address</th>
	<td style="font-weight: bold; background-color: #CCC; padding:3px;" align='center'>User-Agent</th>
	<td style="font-weight: bold; background-color: #CCC; padding:3px;" align='center'>Scope</th>
	<td style="font-weight: bold; background-color: #CCC; padding:3px;" align='center'>Variable</th>
	<td style="font-weight: bold; background-color: #CCC; padding:3px;" align='center'>Content</th>
	<td style="font-weight: bold; background-color: #CCC; padding:3px;" align='center'>Alarm</th>
</tr>
<?php
		preg_match_all( '/<([a-z]+)>([^<]+)<\/[a-z]+>/im', implode($lines), $tokens );
		
		$keys   = $tokens[1];
		$values = $tokens[2];
		
		$xml   = array();
		$xml[] = array();
		$j 	   = 0;
		foreach( $keys as $i => $key ){
			$value = $values[$i];
			if( $key == "rule" ){
				$xml[] = array();
				$j++;
			}
			else{
				$xml[$j][$key] = $value;
			}
		}
		
		$xml = array_reverse($xml);
		
		$j = 0;
		foreach( $xml as $node ){
			if( $j % 2 != 0 ){
				echo "<tr>";
			}
			else{
				echo "<tr style='background-color: #EEE;'>";
			}
					
			foreach( $node as $key => $value ){
				if( $key == "time" ){
					echo "<td style='padding:3px;' align='center'>".strftime("%T",$value)."</td>";
				}
				else if( $key == "address" ){
					echo "<td style='padding:3px;' align='center'><a href='http://whois.domaintools.com/$value' target='_blank'>$value</a></td>";
				}
				else if( $key == "scope" || $key == "variable" || $key == "message" ){
					echo "<td style='padding:3px;' align='center'><b>$value</b></td>";
				}
				else{
					if( strlen($value) > 65 ){
						$value = substr( $value, 0, 61 ) . " ...";
					}
					echo "<td style='padding:3px;'>$value</td>";
				}
			}
			
			$j++;
			echo "</tr>";
		}
		
?>
</table>
</td>
</tr>
</table>
<?php
	}
	else{
		echo "No daily activities.";
	}
}

/*
 * Handle the layout editing tab. 
 */
function wp_sentinel_handle_layouts(){
	if( isset($_POST['update']) ){
		$error = null;
		if( !is_writable(ABSPATH."wp-content/plugins/wp-sentinel/layouts/email.row.layout") ){
			$error = "email.row.layout is not writable!";
		}
		else if( !is_writable(ABSPATH."wp-content/plugins/wp-sentinel/layouts/email.layout") ){
			$error = "email.layout is not writable!";
		}
		else if( !is_writable(ABSPATH."wp-content/plugins/wp-sentinel/layouts/alarm.row.layout") ){
			$error = "alarm.row.layout is not writable!";
		}
		else if( !is_writable(ABSPATH."wp-content/plugins/wp-sentinel/layouts/alarm.layout") ){
			$error = "alarm.layout is not writable!";
		}
		
		foreach( $_POST as $key => $value ){
			$_POST[$key] = html_entity_decode($value);
			if( function_exists('get_magic_quotes_gpc') && get_magic_quotes_gpc() ){
				$_POST[$key] = stripslashes($_POST[$key]);
			}		
			$_POST[$key] = urldecode($_POST[$key]);
		}
		
		if( !file_put_contents( ABSPATH."wp-content/plugins/wp-sentinel/layouts/email.row.layout", $_POST['email_row_layout'] ) ){
			$error = "Error during file update.";
		}
		else if( !file_put_contents( ABSPATH."wp-content/plugins/wp-sentinel/layouts/email.layout", $_POST['email_layout'] ) ){
			$error = "Error during file update.";
		}
		else if( !file_put_contents( ABSPATH."wp-content/plugins/wp-sentinel/layouts/alarm.row.layout", $_POST['alarm_row_layout'] ) ){
			$error = "Error during file update.";
		}
		else if( !file_put_contents( ABSPATH."wp-content/plugins/wp-sentinel/layouts/alarm.layout", $_POST['alarm_layout'] ) ){
			$error = "Error during file update.";
		}
		
		if( $error != null ){
			echo "<div style='background-color: white; color: red; font-size: 13px; padding: 10px; border: 1px dotted gray;'>$error</div><br>";
		}
		else{
			echo "<div style='background-color: white; color: green; font-size: 13px; padding: 10px; border: 1px dotted gray;'>Succesfully updated.</div><br>";
		}
	}
	
	$email_row_layout = @file_get_contents_utf8(ABSPATH."wp-content/plugins/wp-sentinel/layouts/email.row.layout");
	$email_layout     = @file_get_contents_utf8(ABSPATH."wp-content/plugins/wp-sentinel/layouts/email.layout");
	$alarm_row_layout = @file_get_contents_utf8(ABSPATH."wp-content/plugins/wp-sentinel/layouts/alarm.row.layout");
	$alarm_layout     = @file_get_contents_utf8(ABSPATH."wp-content/plugins/wp-sentinel/layouts/alarm.layout");
?>
<form method="POST">
<table width="100%" cellpadding="0" cellspacing="1" style="background-color: white; border: 1px dotted gray; padding: 0;">
	<tr><td><b>Single alarm layout for email notification</b> <em>(content of {ALARM_ROWS} in email layout)</em></td></tr>
	<tr>
	<td>
		Allowed tokens:
		<table cellpadding="0" cellspacing="5">
			<tr>
				<td style="padding: 0px;"><b>{SCOPE}</b></td>
				<td style="padding: 0px;">GET or POST of the variable.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{VARIABLE}</b></td>
				<td style="padding: 0px;">The variable name.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{CONTENT}</b></td>
				<td style="padding: 0px;">The variable content.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{ALARM}</b></td>
				<td style="padding: 0px;">Name of the alarm that was triggered by the request.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{RULE}</b></td>
				<td style="padding: 0px;">Regex that matched the alarm.</td>
			</tr>
		</table>
	</td>
	</tr>
	<tr>
	<td>
		<textarea name="email_row_layout" cols=100 rows=3><?= htmlentities($email_row_layout); ?></textarea>	
	</td>
	</tr>
	
	<tr><td><b>EMail layout</b></td></tr>
	<tr>
	<td>
		Allowed tokens:
		<table cellpadding="0" cellspacing="5">
			<tr>
				<td style="padding: 0px;"><b>{DATE}</b></td>
				<td style="padding: 0px;">Date of the attack.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{ADDRESS}</b></td>
				<td style="padding: 0px;">IP address of the attacker.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{USERNAME}</b></td>
				<td style="padding: 0px;">Wordpress username of the attacker if he's logged.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{REFERER}</b></td>
				<td style="padding: 0px;">HTTP referer.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{USERAGENT}</b></td>
				<td style="padding: 0px;">HTTP user-agent.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{ALARM_ROWS}</b></td>
				<td style="padding: 0px;">Content of the single alarm layouts filled with alarm data.</td>
			</tr>
		</table>
	</td>
	</tr>

	<tr>
	<td>
		<textarea name="email_layout" cols=100 rows=10><?= htmlentities($email_layout); ?></textarea>	
	</td>
	</tr>
	
	<tr><td><b>Single alarm layout for user notification</b> <em>(content of {ALARM_ROWS} in alarm layout)</em></td></tr>
	<tr>
	<td>
		Allowed tokens:
		<table cellpadding="0" cellspacing="5">
			<tr>
				<td style="padding: 0px;"><b>{SCOPE}</b></td>
				<td style="padding: 0px;">GET or POST of the variable.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{VARIABLE}</b></td>
				<td style="padding: 0px;">The variable name.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{CONTENT}</b></td>
				<td style="padding: 0px;">The variable content.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{ALARM}</b></td>
				<td style="padding: 0px;">Name of the alarm that was triggered by the request.</td>
			</tr>
			<tr>
				<td style="padding: 0px;"><b>{RULE}</b></td>
				<td style="padding: 0px;">Regex that matched the alarm.</td>
			</tr>
		</table>
	</td>
	</tr>
	<tr>
	<td>
		<textarea name="alarm_row_layout" cols=100 rows=8><?= htmlentities($alarm_row_layout); ?></textarea>	
	</td>
	</tr>
	
	<tr><td><b>Alarm layout</b></td></tr>
	<tr>
	<td>
		Allowed tokens:
		<table cellpadding="0" cellspacing="5">
			<tr>
				<td style="padding: 0px;"><b>{ALARM_ROWS}</b></td>
				<td style="padding: 0px;">Content of the single alarm layouts filled with alarm data.</td>
			</tr>
		</table>
	</td>
	</tr>
	<tr>
	<td>
		<textarea name="alarm_layout" cols=100 rows=20><?= htmlentities($alarm_layout); ?></textarea>	
	</td>
	</tr>
	
	<tr>
		<td><input class="button-primary" type="submit" value="Update" name="update"/></td> 
	</tr>
</table>
</form>
<?php	
}

/*
 * Handle the settings tab. 
 */
function wp_sentinel_handle_settings(){
	global $wps_config;
	
	$config_path = ABSPATH."wp-content/plugins/wp-sentinel/php/config.php";

	if( isset($_POST['update']) ){
		$error = null;
		if( !is_writable($config_path) ){
			$error = "Configuration file '$config_path' is not writable!";
		}
		
		$tmp 	   = explode( ',', $_POST['whitelist'] );
		$whitelist = array();
		foreach( $tmp as $item ){
			$item = trim($item);
			if( $item != '' ){
				$whitelist[] = "'".trim($item)."'";
			}
		}
		
		$data =
'<?php

$wps_config = array( 
	/* enable or disable wp-sentinel filtering */
	"enabled"      => '.($_POST['enabled'] == 1 ? 1 : 0).',
	/* enable or disable email notifications to the blog administrator */
	"notification" => '.($_POST['notification'] == 1 ? 1 : 0).',
	/* enable or disable logging */
	"logging"      => '.($_POST['logging'] == 1 ? 1 : 0).',
	/* flood delay time in seconds */
	"flood_time"   => '.(INT)$_POST['flood_time'].',
	/* variables to be ignored */
	"whitelist"    => array( '.(implode( ', ', $whitelist )).' )
);
	
?>';	
		
		if( !file_put_contents( $config_path, $data ) ){
			$error = "Error during file update.";
		}
		else{
			include( "php/config.php" );
		}
		
		if( $error != null ){
			echo "<div style='background-color: white; color: red; font-size: 13px; padding: 10px; border: 1px dotted gray;'>$error</div><br>";
		}
		else{
			echo "<div style='background-color: white; color: green; font-size: 13px; padding: 10px; border: 1px dotted gray;'>Succesfully updated.</div><br>";
		}
	}
?>
<form method="POST">
<table width="100%" cellpadding="0" cellspacing="1" style="background-color: white; border: 1px dotted gray; padding: 0;">

<tr>
<td><b>Sentinel Status</b></td> 
<td>
	<input type='radio' name='enabled' <?=($wps_config['enabled'] == 1 ? 'checked' : '');?> value="1"> <font color='green'>Enabled</font>&nbsp;&nbsp;&nbsp;&nbsp;
	<input type='radio' name='enabled' <?=($wps_config['enabled'] != 1 ? 'checked' : '');?> value="0"> <font color='red'>Disabled</font>
</td>
</tr>
<tr>
<td colspan="2" valign="top" style="padding-top: 0px; border-bottom: 1px dotted gray;"><em>Enable or disable the plugin, use carefully!</em></td>
</tr>

<tr>
<td><b>EMail Notifications</b></td> 
<td>
	<input type='radio' name='notification' <?=($wps_config['notification'] == 1 ? 'checked' : '');?> value="1"> Enabled&nbsp;&nbsp;&nbsp;&nbsp;
	<input type='radio' name='notification' <?=($wps_config['notification'] != 1 ? 'checked' : '');?> value="0"> Disabled
</td>
</tr>
<tr>
<td colspan="2" valign="top" style="padding-top: 0px; border-bottom: 1px dotted gray;"><em>Enable or disable email notifications to the administrator.</em></td>
</tr>

<tr>
<td><b>File Logging</b></td> 
<td>
	<input type='radio' name='logging' <?=($wps_config['logging'] == 1 ? 'checked' : '');?> value="1"> <font color='green'>Enabled</font>&nbsp;&nbsp;&nbsp;&nbsp;
	<input type='radio' name='logging' <?=($wps_config['logging'] != 1 ? 'checked' : '');?> value="0"> <font color='red'>Disabled</font>
</td>
</tr>
<tr>
<td colspan="2" valign="top" style="padding-top: 0px; border-bottom: 1px dotted gray;"><em>Enable or disable file logging during an attack.</em></td>
</tr>

<tr>
<td><b>Flood Delay</b></td> 
<td>
	<input type="text" name="flood_time" value="<?= $wps_config['flood_time']; ?>"/>
</td>
</tr>
<tr>
<td colspan="2" valign="top" style="padding-top: 0px; border-bottom: 1px dotted gray;"><em>Delay, in seconds, below which attacks will be considered as flooding and will not be logged.</em></td>
</tr>

<tr>
<td><b>Whitelisted Variables</b></td> 
<td>
	<input type="text" name="whitelist" value="<?= implode( ', ', $wps_config['whitelist'] ); ?>" size="200"/>
</td>
</tr>
<tr>
<td colspan="2" valign="top" style="padding-top: 0px;"><em>Those comma separated variables are not going to be checked by WP-Sentinel.</em></td>
</tr>

<tr>
<td><input class="button-primary" type="submit" value="Update" name="update"/></td> 
<td>
</td>
</tr>

</table>
</form>
<?php
}

function wp_sentinel_menu() {
	add_options_page( 'WP-Sentinel Options', 'WP-Sentinel', 'manage_options', 'wp-sentinel', 'wp_sentinel_options' );
}

function wp_sentinel_options() {
	if( !current_user_can('manage_options') ){
    	wp_die( __('You do not have sufficient permissions to access this page.') );
  	}

?>
<div class="wrap">
<h2>WP-Sentinel Security System</h2>
<?php wp_sentinel_generate_tabs(); ?>
<table class="form-table">
<tr>
<td>
<?php
$tab = isset($_GET['tab']) ? $_GET['tab'] : 'daily';
switch($tab){
	case 'settings' : wp_sentinel_handle_settings(); break;
	case 'layouts'  : wp_sentinel_handle_layouts();  break;

	default 		: wp_sentinel_handle_daily();
}
?>
</td>
</tr>
</table>
</div>
<?
}

?>